#!/usr/bin/env python
#-*- coding:utf-8 -*-
#==============================================================================
#  Copyright (C) 2010 Schaefer & Tobies SuC GmbH.
#  Author: Markus Grimm <mgr@schaefer-tobies.de>
#
#  This file is part of Pyontapi.
#
#  Pyontapi is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyontapi is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pyontapi.  If not, see <http://www.gnu.org/licenses/>.
#
#==============================================================================

""" System API. """

from schtob.pyontapi.api import BaseAPI, NaArgument, NaField
from schtob.pyontapi.api import NaNamedType, NaTypeElement


class System(BaseAPI):

    """ System API class for base system calls. """

    def get_ontapi_version(self):
        """ invoke `system-get-ontapi-version`. """
        fields = (
            NaField({'name': 'major-version', 'type': int}),
            NaField({'name': 'minor-version', 'type': int}),
        )
        return self._filer.do_api_call('system-get-ontapi-version', (),
                                       fields)

    def api_get_elements(self, api_list):
        """ invoke `system-api-get-elements`. """
        args = (
            NaArgument(api_list, {
                'name': 'api-list',
                'type': NaNamedType('api-list-info', [
                            NaTypeElement({'name': '', 'type': str})
                        ]),
                'is-array': True,
            }),
        )
        fields = (
            NaField({'name': 'api-entries', 'is-array': True, 'type':
                NaNamedType('system-api-entry-info', [
                    NaTypeElement({'name': 'name', 'type': str}),
                    NaTypeElement({'name': 'api-elements', 'is-array': True,
                        'type': NaNamedType('system-api-element-info', [
                            NaTypeElement({
                                'name': 'encrypted',
                                'type': str,
                                'is-optional': True,
                            }), NaTypeElement({
                                'name': 'is-nonempty',
                                'type': bool,
                                'is-optional': True,
                            }), NaTypeElement({
                                'name': 'is-optional',
                                'type': bool,
                                'is-optional': True,
                            }), NaTypeElement({
                                'name': 'is-output',
                                'type': bool,
                                'is-optional': True,
                            }), NaTypeElement({
                                'name': 'is-validated',
                                'type': bool,
                                'is-optional': True,
                            }), NaTypeElement({
                                'name': 'name',
                                'type': str,
                            }), NaTypeElement({
                                'name': 'type',
                                'type': str,
                            }),
                    ])})
            ])}),
        )
        return self._filer.do_api_call('system-api-get-elements', args,
                                       fields)

    def api_list(self):
        """ invoke `system-api-list`. """
        fields = (
            NaField({
                'name': 'apis', 'is-array': True,
                'type': NaNamedType('system-api-info', [
                    NaTypeElement({
                        'name': 'is-streaming',
                        'type': bool,
                        'is-optional': True,
                    }),
                    NaTypeElement({
                        'name': 'license',
                        'type': str,
                        'is-optional': True,
                    }),
                    NaTypeElement({
                        'name': 'name',
                        'type': str,
                    }),
                ]),
            }),
        )
        return self._filer.do_api_call('system-api-list', (), fields)

    def get_api_list_types(self):
        """ invoke `system-api-list-types`. """
        fields = (
            NaField({'name': 'type-entries', 'is-array': True,
                'type': NaNamedType('system-api-type-entry-info', [
                NaTypeElement({'name': 'name', 'type': str}),
                NaTypeElement({'name': 'type-elements', 'is-array': True,
                  'type': NaNamedType('system-api-element-info', [
                    NaTypeElement({
                        'name': 'encrypted',
                        'type': str,
                        'is-optional': True,
                    }), NaTypeElement({
                        'name': 'is-nonempty',
                        'type': bool,
                        'is-optional': True,
                    }), NaTypeElement({
                        'name': 'is-optional',
                        'type': bool,
                        'is-optional': True,
                    }), NaTypeElement({
                        'name': 'is-output',
                        'type': bool,
                        'is-optional': True,
                    }), NaTypeElement({
                        'name': 'is-validated',
                        'type': bool,
                        'is-optional': True,
                    }), NaTypeElement({
                        'name': 'name',
                        'type': str,
                    }), NaTypeElement({
                        'name': 'type',
                        'type': str,
                    }),
                ])}),
            ])}),
        )
        return self._filer.do_api_call('system-api-list-types', (), fields)
